@@ -316,6 +316,11 @@ pub enum Value {
     /// A call of a function, such as `call()`
     #[display(fmt = "{}", "_0")]
     FunctionCall(FunctionCall),
+    /// An if expression, such as `if foo then true else false`.
+    /// Only available when the "roblox" feature flag is enabled.
+    #[cfg(feature = "roblox")]
+    #[display(fmt = "{}", "_0")]
+    IfExpression(IfExpression),
     /// A table constructor, such as `{ 1, 2, 3 }`
     #[display(fmt = "{}", "_0")]
     TableConstructor(TableConstructor),
@@ -1266,6 +1271,9 @@ pub enum Call {
 #[derive(Clone, Debug, PartialEq, Node)]
 #[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
 pub struct FunctionBody {
+    #[cfg(feature = "roblox")]
+    generics: Option<GenericDeclaration>,
+
     parameters_parentheses: ContainedSpan,
     parameters: Punctuated<Parameter>,
 
@@ -1284,6 +1292,9 @@ impl FunctionBody {
     /// Returns a new empty FunctionBody
     pub fn new() -> Self {
         Self {
+            #[cfg(feature = "roblox")]
+            generics: None,
+
             parameters_parentheses: ContainedSpan::new(
                 TokenReference::symbol("(").unwrap(),
                 TokenReference::symbol(")").unwrap(),
@@ -1321,6 +1332,14 @@ impl FunctionBody {
         &self.end_token
     }
 
+    /// The generics declared for the function body.
+    /// The `<T, U>` part of `function x<T, U>() end`
+    /// Only available when the "roblox" feature flag is enabled.
+    #[cfg(feature = "roblox")]
+    pub fn generics(&self) -> Option<&GenericDeclaration> {
+        self.generics.as_ref()
+    }
+
     /// The type specifiers of the variables, in the order that they were assigned.
     /// `(foo: number, bar, baz: boolean)` returns an iterator containing:
     /// `Some(TypeSpecifier(number)), None, Some(TypeSpecifier(boolean))`
@@ -1350,6 +1369,12 @@ impl FunctionBody {
         Self { parameters, ..self }
     }
 
+    /// Returns a new FunctionBody with the given generics declaration
+    #[cfg(feature = "roblox")]
+    pub fn with_generics(self, generics: Option<GenericDeclaration>) -> Self {
+        Self { generics, ..self }
+    }
+
     /// Returns a new FunctionBody with the given type specifiers
     #[cfg(feature = "roblox")]
     pub fn with_type_specifiers(self, type_specifiers: Vec<Option<TypeSpecifier>>) -> Self {
@@ -1390,7 +1415,8 @@ impl fmt::Display for FunctionBody {
     fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
         write!(
             formatter,
-            "{}{}{}{}{}{}",
+            "{}{}{}{}{}{}{}",
+            display_option(self.generics.as_ref()),
             self.parameters_parentheses.tokens().0,
             join_type_specifiers(&self.parameters, self.type_specifiers()),
             self.parameters_parentheses.tokens().1,
@@ -1556,21 +1582,12 @@ impl Assignment {
 )]
 #[cfg_attr(
     feature = "roblox",
-    display(
-        fmt = "{}{}{}{}{}",
-        "local_token",
-        "function_token",
-        "name",
-        "display_option(self.generics())",
-        "body"
-    )
+    display(fmt = "{}{}{}{}", "local_token", "function_token", "name", "body")
 )]
 pub struct LocalFunction {
     local_token: TokenReference,
     function_token: TokenReference,
     name: TokenReference,
-    #[cfg(feature = "roblox")]
-    generics: Option<GenericDeclaration>,
     body: FunctionBody,
 }
 
@@ -1581,8 +1598,6 @@ impl LocalFunction {
             local_token: TokenReference::symbol("local ").unwrap(),
             function_token: TokenReference::symbol("function ").unwrap(),
             name,
-            #[cfg(feature = "roblox")]
-            generics: None,
             body: FunctionBody::new(),
         }
     }
@@ -1607,12 +1622,6 @@ impl LocalFunction {
         &self.name
     }
 
-    /// Any generics declared for the function, the `<T, U>` part of `local function x<T, U>() end`
-    #[cfg(feature = "roblox")]
-    pub fn generics(&self) -> Option<&GenericDeclaration> {
-        self.generics.as_ref()
-    }
-
     /// Returns a new LocalFunction with the given `local` token
     pub fn with_local_token(self, local_token: TokenReference) -> Self {
         Self {
@@ -1634,12 +1643,6 @@ impl LocalFunction {
         Self { name, ..self }
     }
 
-    /// Returns a new LocalFunction with the given generics
-    #[cfg(feature = "roblox")]
-    pub fn with_generics(self, generics: Option<GenericDeclaration>) -> Self {
-        Self { generics, ..self }
-    }
-
     /// Returns a new LocalFunction with the given function body
     pub fn with_body(self, body: FunctionBody) -> Self {
         Self { body, ..self }
@@ -1934,19 +1937,11 @@ impl FunctionName {
 )]
 #[cfg_attr(
     feature = "roblox",
-    display(
-        fmt = "{}{}{}{}",
-        "function_token",
-        "name",
-        "display_option(self.generics())",
-        "body"
-    )
+    display(fmt = "{}{}{}", "function_token", "name", "body")
 )]
 pub struct FunctionDeclaration {
     function_token: TokenReference,
     name: FunctionName,
-    #[cfg(feature = "roblox")]
-    generics: Option<GenericDeclaration>,
     body: FunctionBody,
 }
 
@@ -1956,8 +1951,6 @@ impl FunctionDeclaration {
         Self {
             function_token: TokenReference::symbol("function ").unwrap(),
             name,
-            #[cfg(feature = "roblox")]
-            generics: None,
             body: FunctionBody::new(),
         }
     }
@@ -1977,12 +1970,6 @@ impl FunctionDeclaration {
         &self.name
     }
 
-    /// Any generics declared for the function
-    #[cfg(feature = "roblox")]
-    pub fn generics(&self) -> Option<&GenericDeclaration> {
-        self.generics.as_ref()
-    }
-
     /// Returns a new FunctionDeclaration with the given `function` token
     pub fn with_function_token(self, function_token: TokenReference) -> Self {
         Self {
@@ -1996,12 +1983,6 @@ impl FunctionDeclaration {
         Self { name, ..self }
     }
 
-    /// Returns a new FunctionDeclaration with the given generics
-    #[cfg(feature = "roblox")]
-    pub fn with_generics(self, generics: Option<GenericDeclaration>) -> Self {
-        Self { generics, ..self }
-    }
-
     /// Returns a new FunctionDeclaration with the given function body
     pub fn with_body(self, body: FunctionBody) -> Self {
         Self { body, ..self }
@@ -2055,6 +2036,27 @@ impl BinOp {
     pub fn is_right_associative(&self) -> bool {
         matches!(*self, BinOp::Caret(_) | BinOp::TwoDots(_))
     }
+
+    /// The token associated with the operator
+    pub fn token(&self) -> &TokenReference {
+        match self {
+            BinOp::And(token)
+            | BinOp::Caret(token)
+            | BinOp::GreaterThan(token)
+            | BinOp::GreaterThanEqual(token)
+            | BinOp::LessThan(token)
+            | BinOp::LessThanEqual(token)
+            | BinOp::Minus(token)
+            | BinOp::Or(token)
+            | BinOp::Percent(token)
+            | BinOp::Plus(token)
+            | BinOp::Slash(token)
+            | BinOp::Star(token)
+            | BinOp::TildeEqual(token)
+            | BinOp::TwoDots(token)
+            | BinOp::TwoEqual(token) => token,
+        }
+    }
 }
 
 make_op!(UnOp,
@@ -2072,6 +2074,13 @@ impl UnOp {
     pub fn precedence(&self) -> u8 {
         7
     }
+
+    /// The token associated with the operator
+    pub fn token(&self) -> &TokenReference {
+        match self {
+            UnOp::Minus(token) | UnOp::Not(token) | UnOp::Hash(token) => token,
+        }
+    }
 }
 
 /// An error that occurs when creating the ast *after* tokenizing
@@ -2135,63 +2144,63 @@ impl Ast {
     /// UnexpectedToken error will be returned.
     pub fn from_tokens(tokens: Vec<Token>) -> Result<Ast, AstError> {
         if *tokens.last().ok_or(AstError::Empty)?.token_type() != TokenType::Eof {
-            Err(AstError::NoEof)
-        } else {
-            let mut tokens = extract_token_references(tokens);
-            let mut state = ParserState::new(&tokens);
+            return Err(AstError::NoEof);
+        }
 
-            if tokens
-                .iter()
-                .filter(|token| !token.token_type().is_trivia())
-                .count()
-                == 1
-            {
-                // Entirely comments/whitespace
-                return Ok(Ast {
-                    nodes: Block {
-                        stmts: Vec::new(),
-                        last_stmt: None,
-                    },
-                    eof: tokens.pop().expect(
-                        "(internal full-moon error) No EOF in tokens after checking for EOF.",
-                    ),
-                });
-            }
+        let mut tokens = extract_token_references(tokens);
+        let mut state = ParserState::new(&tokens);
 
-            // ParserState has to have at least 2 tokens, the last being an EOF, thus unwrap() can't fail
-            if state.peek().token_type().is_trivia() {
-                state = state.advance().unwrap();
-            }
+        if tokens
+            .iter()
+            .filter(|token| !token.token_type().is_trivia())
+            .count()
+            == 1
+        {
+            // Entirely comments/whitespace
+            return Ok(Ast {
+                nodes: Block {
+                    stmts: Vec::new(),
+                    last_stmt: None,
+                },
+                eof: tokens
+                    .pop()
+                    .expect("(internal full-moon error) No EOF in tokens after checking for EOF."),
+            });
+        }
 
-            match parsers::ParseBlock.parse(state) {
-                Ok((state, block)) => {
-                    if state.index == tokens.len() - 1 {
-                        Ok(Ast {
-                            nodes: block,
-                            eof: tokens.pop().expect(
-                                "(internal full-moon error) No EOF in tokens after checking for EOF."
-                            ),
-                        })
-                    } else {
-                        Err(AstError::UnexpectedToken {
-                            token: state.peek().token.clone(),
-                            additional: Some(Cow::Borrowed("leftover token")),
-                        })
-                    }
-                }
+        // ParserState has to have at least 2 tokens, the last being an EOF, thus unwrap() can't fail
+        if state.peek().token_type().is_trivia() {
+            state = state.advance().unwrap();
+        }
 
-                Err(InternalAstError::NoMatch) => Err(AstError::UnexpectedToken {
-                    token: state.peek().token.clone(),
-                    additional: None,
-                }),
-
-                Err(InternalAstError::UnexpectedToken { token, additional }) => {
+        match parsers::ParseBlock.parse(state) {
+            Ok((state, block)) => {
+                if state.index == tokens.len() - 1 {
+                    Ok(Ast {
+                        nodes: block,
+                        eof: tokens.pop().expect(
+                            "(internal full-moon error) No EOF in tokens after checking for EOF.",
+                        ),
+                    })
+                } else {
                     Err(AstError::UnexpectedToken {
-                        token: token.token,
-                        additional,
+                        token: state.peek().token.clone(),
+                        additional: Some(Cow::Borrowed("leftover token")),
                     })
                 }
             }
+
+            Err(InternalAstError::NoMatch) => Err(AstError::UnexpectedToken {
+                token: state.peek().token.clone(),
+                additional: None,
+            }),
+
+            Err(InternalAstError::UnexpectedToken { token, additional }) => {
+                Err(AstError::UnexpectedToken {
+                    token: token.token,
+                    additional,
+                })
+            }
         }
     }
 
@@ -2383,10 +2392,10 @@ mod tests {
                 parentheses: ContainedSpan::new(token.clone(), token.clone()),
             },
         );
-        NumericFor::new(token.clone(), expression.clone(), expression.clone());
+        NumericFor::new(token, expression.clone(), expression.clone());
         Repeat::new(expression.clone());
         Return::new();
         TableConstructor::new();
-        While::new(expression.clone());
+        While::new(expression);
     }
 }
