@@ -30,10 +30,10 @@ symbols!(
     True => "true",
     Until => "until",
     While => "while",
-    // TODO: This only is valid in Lua 5.2
+    #[cfg(feature = "lua52")]
     Goto => "goto",
 
-    // TODO: This only is valid in Roblox
+    #[cfg(feature = "roblox")]
     PlusEqual => "+=",
     MinusEqual => "-=",
     StarEqual => "*=",
@@ -41,11 +41,11 @@ symbols!(
     PercentEqual => "%=",
     CaretEqual => "^=",
     TwoDotsEqual => "..=",
-    // TODO: This only is valid in Roblox
+    #[cfg(feature = "roblox")]
     Ampersand => "&",
-    // TODO: This only is valid in Roblox
+    #[cfg(feature = "roblox")]
     ThinArrow => "->",
-    // TODO: This only is valid in Roblox and Lua 5.2
+    #[cfg(any(feature = "roblox", feature = "lua52"))]
     TwoColons => "::",
     Caret => "^",
     Colon => ":",
@@ -65,10 +65,10 @@ symbols!(
     LessThan => "<",
     Minus => "-",
     Percent => "%",
-    // TODO: This only is valid in Roblox
+    #[cfg(feature = "roblox")]
     Pipe => "|",
     Plus => "+",
-    // TODO: This only is valid in Roblox
+    #[cfg(feature = "roblox")]
     QuestionMark => "?",
     RightBrace => "}",
     RightBracket => "]",
@@ -304,9 +304,9 @@ impl fmt::Display for Token {
                 quote_type,
             } => {
                 if let Some(blocks) = multi_line {
-                    format!("[{0}[{1}]{0}]", "=".repeat(*blocks), literal.to_string())
+                    format!("[{0}[{1}]{0}]", "=".repeat(*blocks), literal)
                 } else {
-                    format!("{0}{1}{0}", quote_type.to_string(), literal.to_string())
+                    format!("{0}{1}{0}", quote_type.to_string(), literal)
                 }
             }
             Symbol { symbol } => symbol.to_string(),
@@ -424,11 +424,11 @@ impl TokenReference {
 
         let mut symbol_text = String::new();
         while let Some(character) = chars.peek() {
-            if !character.is_ascii_whitespace() {
-                symbol_text.push(chars.next().unwrap());
-            } else {
+            if character.is_ascii_whitespace() {
                 break;
             }
+
+            symbol_text.push(chars.next().unwrap());
         }
 
         let symbol = Symbol::from_str(&symbol_text)
@@ -702,6 +702,10 @@ peg::parser! {
             = line:$("#!" (!line_ending() [_])* line_ending())
               {TokenType::Shebang{line:line.into()}.into()}
 
+        pub(super) rule utf8_bom() -> RawToken
+            = chars:$("\u{feff}") // A BOM sequence in UTF8 "\xEF\xBB\xBF"
+            { TokenType::Whitespace { characters:chars.into() }.into() }
+
         pub(super) rule identifier() -> RawToken
             = id:$(['_'|'a'..='z'|'A'..='Z'] ['_'|'a'..='z'|'A'..='Z'|'0'..='9']*)
               { match parse_keyword(id) {
@@ -768,12 +772,16 @@ peg::parser! {
             / identifier()
 
         pub(crate) rule tokens() -> Vec<(RawToken, usize)>
-            = shebang:(shebang:shebang() pos:position!() {(shebang,pos)})?
+            = bom:(bom:utf8_bom() pos:position!() {(bom,pos)})?
+              shebang:(shebang:shebang() pos:position!() {(shebang,pos)})?
               body:( token:token() pos:position!() {(token,pos)})*
               {
                   let mut body = body;
                   if let Some(shebang) = shebang {
-                      body.insert(0, shebang)
+                      body.insert(0, shebang);
+                  }
+                  if let Some(bom) = bom {
+                      body.insert(0, bom);
                   }
                   body
               }
@@ -1179,6 +1187,22 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_rule_bom() {
+        let bom = String::from_utf8(b"\xEF\xBB\xBF".to_vec()).unwrap();
+        test_rule!(
+            utf8_bom(&bom),
+            TokenType::Whitespace {
+                characters: ShortString::new(&bom),
+            }
+        );
+        // Don't recognize if not in the beggining.
+        test_rule!(
+            &format!("#!/usr/bin/env lua\n {}", bom),
+            TokenizerErrorType::UnexpectedToken('\u{feff}')
+        );
+    }
+
     #[test]
     fn test_new_line_on_same_line() {
         assert_eq!(
